@@ -1,4 +1,13 @@
-#    Copyright (c) 2007-2010 Dominique Dumont.
+#
+# This file is part of Config-Model-Itself
+#
+# This software is Copyright (c) 2011 by Dominique Dumont.
+#
+# This is free software, licensed under:
+#
+#   The GNU Lesser General Public License, Version 2.1, February 1999
+#
+#    Copyright (c) 2007-2011 Dominique Dumont.
 #
 #    This file is part of Config-Model-Itself.
 #
@@ -17,6 +26,9 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::Itself ;
+BEGIN {
+  $Config::Model::Itself::VERSION = '1.225';
+}
 
 use strict;
 use warnings ;
@@ -28,9 +40,7 @@ use File::Find ;
 use File::Path ;
 use File::Basename ;
 
-our $VERSION = '1.215';
-
-my $logger = Log::Log4perl::get_logger(__PACKAGE__);
+my $logger = Log::Log4perl::get_logger("Backend::Itself");
 
 =head1 NAME
 
@@ -55,7 +65,7 @@ Config::Model::Itself - Model editor for Config::Model
  $rw_obj -> read_all( conf_dir => '/path/to/model_files') ;
 
  # For Curses UI prepare a call-back to write model
- my $wr_back = sub { $rw_obj->write_all(conf_dir => '/path/to/model_files');
+ my $wr_back = sub { $rw_obj->write_all(model_dir => '/path/to/model_files');
 
  # create Curses user interface
  my $dialog = Config::Model::CursesUI-> new
@@ -138,95 +148,104 @@ sub read_all {
     my %args = @_ ;
     my $model_obj = $self->{model_object};
     my $dir = $args{model_dir} 
-      || croak __PACKAGE__," read_all: undefined config dir";
+      || croak __PACKAGE__," read_all: undefined model dir";
     my $model = $args{root_model} 
       || croak __PACKAGE__," read_all: undefined root_model";
     my $force_load = $args{force_load} || 0 ;
     my $legacy = $args{legacy} ;
 
     unless (-d $dir ) {
-	croak __PACKAGE__," read_all: unknown config dir $dir";
+        croak __PACKAGE__," read_all: unknown model dir $dir";
     }
 
+    my $root_model_file = $model ;
+    $root_model_file =~ s!::!/!g ;
+    
     my @files ;
     my $wanted = sub { 
-	my $n = $File::Find::name ;
-	push @files, $n if (-f $_ and not /~$/ 
-			    and $n !~ /CVS/
-			    and $n !~ m!.svn!
-			    and $n =~ /\b$model/
-			   ) ;
+        my $n = $File::Find::name ;
+        push @files, $n if (-f $_ and not /~$/ 
+                            and $n !~ /CVS/
+                            and $n !~ m!.(svn|orig|pod)$!
+                            and $n =~ m!$dir/$root_model_file!
+                           ) ;
     } ;
     find ($wanted, $dir ) ;
 
     my $i = $model_obj->instance ;
     my %read_models ;
+    my %pod_data ;
     my %class_file_map ;
 
     for my $file (@files) {
-	$logger->info("loading config file $file");
+        $logger->info("loading config file $file");
 
-	# now apply some translation to read model
-	# - translate legacy warp parameters
-	# - expand elements name
-	my $tmp_model = Config::Model -> new( skip_include => 1, legacy => $legacy ) ;
-	my @models = $tmp_model -> load ( 'Tmp' , $file ) ;
+        # now apply some translation to read model
+        # - translate legacy warp parameters
+        # - expand elements name
+        my $tmp_model = Config::Model -> new( skip_include => 1, legacy => $legacy ) ;
+        my @models = $tmp_model -> load ( 'Tmp' , $file ) ;
 
-	my $rel_file = $file ;
-	$rel_file =~ s/^$dir\/?//;
-	die "wrong reg_exp" if $file eq $rel_file ;
-	$class_file_map{$rel_file} = \@models ;
+        my $rel_file = $file ;
+        $rel_file =~ s/^$dir\/?//;
+        die "wrong reg_exp" if $file eq $rel_file ;
+        $class_file_map{$rel_file} = \@models ;
 
-	# - move experience, description and level status into parameter info.
-	foreach my $model_name (@models) {
-	    # no need to dclone model as Config::Model object is temporary
-	    my $new_model =  $tmp_model -> get_model( $model_name ) ;
+        # - move experience, description and level status into parameter info.
+        foreach my $model_name (@models) {
+            # no need to dclone model as Config::Model object is temporary
+            my $new_model =  $tmp_model -> get_model( $model_name ) ;
 
-	    foreach my $item (qw/description summary level experience status/) {
-		foreach my $elt_name (keys %{$new_model->{element}}) {
-		    my $moved_data = delete $new_model->{$item}{$elt_name}  ;
-		    next unless defined $moved_data ;
-		    $new_model->{element}{$elt_name}{$item} = $moved_data ; 
-		}
-		delete $new_model->{$item} ;
-	    }
+            foreach my $item (qw/description summary level experience status/) {
+                foreach my $elt_name (keys %{$new_model->{element}}) {
+                    my $moved_data = delete $new_model->{$item}{$elt_name}  ;
+                    next unless defined $moved_data ;
+                    $new_model->{element}{$elt_name}{$item} = $moved_data ; 
+                }
+                delete $new_model->{$item} ;
+            }
 
-	    # cleanup
+            # Since accept specs and elements are stored in a ordered hash,
+            # load_data expects a array ref instead of a hash ref.
+            # Build this array ref taking the order into
+            # account
+            foreach my $what (qw/element accept/) {
+                my $list  = delete $new_model -> {$what.'_list'} ;
+                my $h     = delete $new_model -> {$what} ;
+                $new_model -> {$what} = [] ;
+                map { 
+                    push @{$new_model->{$what}}, $_, $h->{$_} 
+                } @$list ;
+            }
 
-	    # Since the element are stored in a ordered hash,
-	    # load_data expects a array ref instead of a hash ref.
-	    # Build this array ref taking the element order into
-	    # account
-	    my $list  = delete $new_model -> {element_list} ;
-	    my $elt_h = delete $new_model -> {element} ;
-	    $new_model -> {element} = [] ;
-	    map { 
-		push @{$new_model->{element}}, $_, $elt_h->{$_} 
-	    } @$list ;
-
-
-	    # remove hash key with undefined values
-	    map { delete $new_model->{$_} unless defined $new_model->{$_} 
-		                          and $new_model->{$_} ne ''
-	      } keys %$new_model ;
-	    $read_models{$model_name} = $new_model ;
-	}
+            # remove hash key with undefined values
+            map { delete $new_model->{$_} unless defined $new_model->{$_} 
+                                          and $new_model->{$_} ne ''
+              } keys %$new_model ;
+            $read_models{$model_name} = $new_model ;
+        }
 
     }
 
     # Create all classes listed in %read_models to avoid problems with
     # include statement while calling load_data
     my $class_element = $model_obj->fetch_element('class') ;
-    map { $class_element->fetch_with_id($_) } keys %read_models ;
+    map { $class_element->fetch_with_id($_) } sort keys %read_models ;
 
-    #print Dumper \@read_models ;
     #require Tk::ObjScanner; Tk::ObjScanner::scan_object(\%read_models) ;
-    $model_obj->instance->push_no_value_check(qw/store fetch type/) if $force_load;
 
     $logger->info("loading all extracted data in Config::Model::Itself");
-    $model_obj->load_data( {class => \%read_models} ) ;
+    # load with a array ref to avoid warnings about missing order
+    $model_obj->load_data( {class => [ %read_models ] }, undef, $force_load ? 'no' : 'yes' ) ;
 
-    $model_obj->instance->pop_no_value_check() if $force_load;
+    # load annotations
+    for my $file (@files) {
+        $logger->info("loading annotations from file $file");
+        my $fh = IO::File->new($file) || die "Can't open $file: $!" ;
+        my @lines = $fh->getlines ;  
+        $fh->close;
+        $model_obj->load_pod_annotation(join('',@lines)) ;
+    }
 
     return $self->{map} = \%class_file_map ;
 }
@@ -239,8 +258,12 @@ sub get_perl_data_model{
     my $class_name = $args{class_name}
       || croak __PACKAGE__," read: undefined class name";
 
-    my $class_elt = $model_obj->fetch_element('class')
-      ->fetch_with_id($class_name) ;
+    my $class_element = $model_obj->fetch_element('class') ; 
+
+    # skip if class was deleted during edition
+    return unless $class_element->defined($class_name) ;
+    
+    my $class_elt = $class_element -> fetch_with_id($class_name) ;
 
     my $model = $class_elt->dump_as_data ;
 
@@ -278,64 +301,77 @@ sub write_all {
     my %args = @_ ;
     my $model_obj = $self->{model_object} ;
     my $dir = $args{model_dir} 
-      || croak __PACKAGE__," write_all: undefined config dir";
+      || croak __PACKAGE__," write_all: undefined model_dir";
 
     my $map = $self->{map} ;
 
     unless (-d $dir ) {
-	mkpath($dir,0, 0755) || die "Can't mkpath $dir:$!";
+        mkpath($dir,0, 0755) || die "Can't mkpath $dir:$!";
     }
 
-    my $i = $model_obj->instance ;
+    #my $i = $model_obj->instance ;
 
     # get list of all classes loaded by the editor
     my %loaded_classes 
       = map { ($_ => 1); } 
-	$model_obj->fetch_element('class')->get_all_indexes ;
+        $model_obj->fetch_element('class')->get_all_indexes ;
 
     # remove classes that are listed in map
     foreach my $file (keys %$map) {
-	foreach my $class_name (@{$map->{$file}}) {
-	    delete $loaded_classes{$class_name} ;
-	}
+        foreach my $class_name (@{$map->{$file}}) {
+            delete $loaded_classes{$class_name} ;
+        }
     }
 
     # add remaining classes in map
     my %new_map =  map { 
-	my $f = $_; 
-	$f =~ s!::!/!g; 
-	("$f.pl" => [ $_ ]) ;
+        my $f = $_; 
+        $f =~ s!::!/!g; 
+        ("$f.pl" => [ $_ ]) ;
     } keys %loaded_classes ;
 
     my %map_to_write = (%$map,%new_map) ;
 
     foreach my $file (keys %map_to_write) {
-	$logger->info("writing config file $file");
+        $logger->info("writing config file $file");
 
-	my @data ;
+        my @data ;
+        my @notes ;
 
-	foreach my $class_name (@{$map_to_write{$file}}) {
-	    $logger->info("writing class $class_name");
-	    my $model 
-	      = $self-> get_perl_data_model(class_name   => $class_name) ;
-	    push @data, $model ;
-	    # remove class name from above list
-	    delete $loaded_classes{$class_name} ;
-	}
+        foreach my $class_name (@{$map_to_write{$file}}) {
+            $logger->info("writing class $class_name");
+            my $model 
+              = $self-> get_perl_data_model(class_name => $class_name) ;
+            push @data, $model if defined $model;
+            
+            my $node = $self->{model_object}->grab("class:".$class_name) ;
+            push @notes, $node->dump_annotations_as_pod ;
+            # remove class name from above list
+            delete $loaded_classes{$class_name} ;
+        }
 
-	my $wr_file = "$dir/$file" ;
-	my $wr_dir  = dirname($wr_file) ;
-	unless (-d $wr_dir ) {
-	    mkpath($wr_dir,0, 0755) || die "Can't mkpath $wr_dir:$!";
-	}
+        my $wr_file = "$dir/$file" ;
+        my $wr_dir  = dirname($wr_file) ;
+        unless (-d $wr_dir ) {
+            mkpath($wr_dir,0, 0755) || die "Can't mkpath $wr_dir:$!";
+        }
 
-	open (WR, ">$wr_file") || croak "Cannot open file $wr_file:$!" ;
-	my $dumper = Data::Dumper->new([\@data]) ;
-	$dumper->Terse(1) ;
-	print WR $dumper->Dump , ";\n";
-	close WR ;
+        my $wr = IO::File->new ($wr_file,'>') || croak "Cannot open file $wr_file:$!" ;
+
+        my $dumper = Data::Dumper->new([\@data]) ;
+        $dumper->Indent(1) ; # avoid too deep indentation
+        $dumper->Terse(1) ; # allow unnamed variables in dump
+
+        my $dump = $dumper->Dump;
+        # munge pod text embedded in values to avoid spurious pod formatting
+        $dump =~ s/\n=/\n'.'=/g ;
+
+        $wr->print ( $dump , ";\n\n");
+
+        $wr->print( join("\n",@notes )) ;
+
+        $wr->close ;
     }
-
 }
 
 
@@ -353,7 +389,7 @@ sub list_class_element {
     my $res = '';
     my $meta_class = $self->{model_object}->fetch_element('class') ;
     foreach my $class_name ($meta_class->get_all_indexes ) {
-	$res .= $self->list_one_class_element($class_name) ;
+        $res .= $self->list_one_class_element($class_name) ;
     }
     return $res ;
 }
@@ -373,18 +409,18 @@ sub list_one_class_element {
     my $inc_after = $meta_class->grab_value('include_after') ;
 
     if (@include and not defined $inc_after) {
-	map { $res .= $self->list_one_class_element($_,$pad.'  ') ;} @include ;
+        map { $res .= $self->list_one_class_element($_,$pad.'  ') ;} @include ;
     }
 
     return $res unless @elts ;
 
     foreach my $elt_name ( @elts) {
-	my $type = $meta_class->grab_value("element:$elt_name type") ;
+        my $type = $meta_class->grab_value("element:$elt_name type") ;
 
-	$res .= $pad."  - $elt_name ($type)\n";
-	if (@include and defined $inc_after and $inc_after eq $elt_name) {
-	    map { $res .=$self->list_one_class_element($_,$pad.'  ') ;} @include ;
-	}
+        $res .= $pad."  - $elt_name ($type)\n";
+        if (@include and defined $inc_after and $inc_after eq $elt_name) {
+            map { $res .=$self->list_one_class_element($_,$pad.'  ') ;} @include ;
+        }
     }
     return $res ;
 }
@@ -415,38 +451,41 @@ sub get_dot_diagram {
 
     my $meta_class = $self->{model_object}->fetch_element('class') ;
     foreach my $class_name ($meta_class->get_all_indexes ) {
-	my $c_model = $self->{model_object}->config_model->get_raw_model($class_name);
-	my $elts = $c_model->{element} || []; # array ref
+        my $c_model = $self->{model_object}->config_model->get_raw_model($class_name);
+        my $elts = $c_model->{element} || []; # array ref
 
-	my $d_class = $class_name ;
-	$d_class =~ s/::/__/g;
+        my $d_class = $class_name ;
+        $d_class =~ s/::/__/g;
 
-	my $elt_list = '';
-	my $use = '';
-	for (my $idx = 0; $idx < @$elts; $idx += 2) {
-	    my $elt_info = $elts->[$idx] ;
-	    my @elt_names = ref $elt_info ? @$elt_info : ($elt_info) ;
-	    my $type = $elts->[$idx+1]{type} ;
+        my $elt_list = '';
+        my $use = '';
+        for (my $idx = 0; $idx < @$elts; $idx += 2) {
+            my $elt_info = $elts->[$idx] ;
+            my @elt_names = ref $elt_info ? @$elt_info : ($elt_info) ;
+            my $type = $elts->[$idx+1]{type} ;
 
-	    foreach my $elt_name (@elt_names) {
-		$elt_list .= "- $elt_name ($type)\\n";
-		$use .= $self->scan_used_class($d_class,$elt_name,
-					       $elts->[$idx+1]);
-	    }
-	}
+            foreach my $elt_name (@elt_names) {
+                my $of = '';
+                my $cargo = $elts->[$idx+1]{cargo}{type} ;
+                $of = " of $cargo" if defined $cargo ;
+                $elt_list .= "- $elt_name ($type$of)\\n";
+                $use .= $self->scan_used_class($d_class,$elt_name,
+                                               $elts->[$idx+1]);
+            }
+        }
 
-	$dot .= $d_class 
+        $dot .= $d_class 
              .  qq! [shape=box label="$class_name\\n$elt_list"];\n!
-	     .  $use . "\n";
+             .  $use . "\n";
 
-	my $include = $c_model->{include} ;
-	if (defined $include) {
-	    my $inc_ref = ref $include ? $include : [ $include ] ;
-	    foreach my $t (@$inc_ref) {
-		$t =~ s/::/__/g;
-		$dot.= qq!$d_class -> $t ;\n!;
-	    }
-	}
+        my $include = $c_model->{include} ;
+        if (defined $include) {
+            my $inc_ref = ref $include ? $include : [ $include ] ;
+            foreach my $t (@$inc_ref) {
+                $t =~ s/::/__/g;
+                $dot.= qq!$d_class -> $t ;\n!;
+            }
+        }
     }
 
     $dot .="}\n";
@@ -459,20 +498,20 @@ sub scan_used_class {
     my $res = '' ;
 
     if (ref($ref) eq 'HASH') {
-	foreach my $k (keys %$ref) {
-	    my $v = $ref->{$k} ;
-	    if ($k eq 'config_class_name') {
-		$v =~ s/::/__/g;
-		$res .= qq!$d_class -> $v !
-		      . qq![ style=dashed, label="$elt_name" ];\n!;
-	    }
-	    if (ref $v) {
-		$res .= $self->scan_used_class($d_class,$elt_name,$v);
-	    }
-	}
+        foreach my $k (keys %$ref) {
+            my $v = $ref->{$k} ;
+            if ($k eq 'config_class_name') {
+                $v =~ s/::/__/g;
+                $res .= qq!$d_class -> $v !
+                      . qq![ style=dashed, label="$elt_name" ];\n!;
+            }
+            if (ref $v) {
+                $res .= $self->scan_used_class($d_class,$elt_name,$v);
+            }
+        }
     }
     elsif (ref($ref) eq 'ARRAY') {
-	map {$res .= $self->scan_used_class($d_class,$elt_name,$_);} @$ref ;
+        map {$res .= $self->scan_used_class($d_class,$elt_name,$_);} @$ref ;
     }
     return $res ;
 }
@@ -487,7 +526,7 @@ Dominique Dumont, (ddumont at cpan dot org)
 
 =head1 COPYRIGHT
 
-Copyright (C) 2007-2010 by Dominique Dumont
+Copyright (C) 2007-2011 by Dominique Dumont
 
 =head1 LICENSE
 
